@@ -216,11 +216,6 @@ static int _set_cond(int *start, int argc, char *argv[],
 					" Bad Preempt Mode given: %s\n",
 					argv[i]);
 				exit_code = 1;
-			} else if (qos_cond->preempt_mode ==
-				   PREEMPT_MODE_SUSPEND) {
-				printf("PreemptType and PreemptMode "
-					"values incompatible\n");
-				exit_code = 1;
 			} else
 				set = 1;
 		} else {
@@ -244,6 +239,9 @@ static int _set_rec(int *start, int argc, char *argv[],
 	int end = 0;
 	int command_len = 0;
 	int option = 0;
+	uint64_t tmp64;
+	char *tmp_char = NULL;
+	uint32_t tres_flags = TRES_STR_FLAG_SORT_ID | TRES_STR_FLAG_REPLACE;
 
 	for (i=(*start); i<argc; i++) {
 		end = parse_option_end(argv[i]);
@@ -268,18 +266,16 @@ static int _set_rec(int *start, int argc, char *argv[],
 					   MAX(command_len, 1))) {
 			if (name_list)
 				slurm_addto_char_list(name_list, argv[i]+end);
-		} else if (!strncasecmp (argv[i], "Description",
+		} else if (!qos)
+			continue;
+		else if (!strncasecmp (argv[i], "Description",
 					 MAX(command_len, 1))) {
-			if (!qos)
-				continue;
 			if (!qos->description)
 				qos->description =
 					strip_quotes(argv[i]+end, NULL, 1);
 			set = 1;
 		} else if (!strncasecmp (argv[i], "Flags",
 					 MAX(command_len, 2))) {
-			if (!qos)
-				continue;
 			qos->flags = str_2_qos_flags(argv[i]+end, option);
 			if (qos->flags == QOS_FLAG_NOTSET) {
 				char *tmp_char = NULL;
@@ -297,66 +293,140 @@ static int _set_rec(int *start, int argc, char *argv[],
 				set = 1;
 		} else if (!strncasecmp (argv[i], "GraceTime",
 					 MAX(command_len, 3))) {
-			if (!qos)
-				continue;
 			if (get_uint(argv[i]+end, &qos->grace_time,
 			             "GraceTime") == SLURM_SUCCESS) {
 				set = 1;
 			}
 		} else if (!strncasecmp (argv[i], "GrpCPUMins",
 					 MAX(command_len, 7))) {
-			if (!qos)
-				continue;
 			if (get_uint64(argv[i]+end,
-				       &qos->grp_cpu_mins,
-				       "GrpCPUMins") == SLURM_SUCCESS)
+				       &tmp64,
+				       "GrpCPUMins") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_CPU, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->grp_tres_mins, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "GrpCPURunMins",
 					 MAX(command_len, 7))) {
-			if (!qos)
-				continue;
-			if (get_uint64(argv[i]+end, &qos->grp_cpu_run_mins,
-				       "GrpCPURunMins") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "GrpCPURunMins") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_CPU, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->grp_tres_run_mins, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "GrpCPUs",
 					 MAX(command_len, 7))) {
-			if (!qos)
-				continue;
-			if (get_uint(argv[i]+end, &qos->grp_cpus,
-				     "GrpCPUs") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "GrpCPUs") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_CPU, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->grp_tres, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "GrpJobs",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
 			if (get_uint(argv[i]+end, &qos->grp_jobs,
 			    "GrpJobs") == SLURM_SUCCESS)
 				set = 1;
 		} else if (!strncasecmp (argv[i], "GrpMemory",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
-			if (get_uint(argv[i]+end, &qos->grp_mem,
-				     "GrpMemory") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "GrpMemory") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_MEM, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->grp_tres, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "GrpNodes",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
-			if (get_uint(argv[i]+end, &qos->grp_nodes,
-			    "GrpNodes") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "GrpNodes") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_NODE, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->grp_tres, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "GrpSubmitJobs",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
 			if (get_uint(argv[i]+end, &qos->grp_submit_jobs,
 			    "GrpSubmitJobs") == SLURM_SUCCESS)
 				set = 1;
+		} else if (!strncasecmp(argv[i], "GrpTRES",
+					MAX(command_len, 7))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(
+					db_conn, &tres_cond);
+			}
+
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->grp_tres, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
+		} else if (!strncasecmp(argv[i], "GrpTRESMins",
+					MAX(command_len, 8))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(db_conn,
+							       &tres_cond);
+			}
+
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->grp_tres_mins, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
+		} else if (!strncasecmp(argv[i], "GrpTRESRunMins",
+					MAX(command_len, 8))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(
+					db_conn, &tres_cond);
+			}
+
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->grp_tres_run_mins, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "GrpWall",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
 			mins = time_str2mins(argv[i]+end);
 			if (mins != NO_VAL) {
 				qos->grp_wall	= (uint32_t) mins;
@@ -369,60 +439,170 @@ static int _set_rec(int *start, int argc, char *argv[],
 			}
 		} else if (!strncasecmp (argv[i], "MaxCPUMinsPerJob",
 					 MAX(command_len, 7))) {
-			if (!qos)
-				continue;
 			if (get_uint64(argv[i]+end,
-				       &qos->max_cpu_mins_pj,
-				       "MaxCPUMins") == SLURM_SUCCESS)
+				       &tmp64,
+				       "MaxCPUMins") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_CPU, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_mins_pj, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "MaxCPUsPerJob",
 					 MAX(command_len, 7))) {
-			if (!qos)
-				continue;
-			if (get_uint(argv[i]+end, &qos->max_cpus_pj,
-			    "MaxCPUs") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "MaxCPUs") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_CPU, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_pj, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "MaxCPUsPerUser",
 					 MAX(command_len, 11))) {
-			if (!qos)
-				continue;
-			if (get_uint(argv[i]+end, &qos->max_cpus_pu,
-			    "MaxCPUsPerUser") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "MaxCPUsPerUser") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_CPU, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_pu, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "MaxJobsPerUser",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
 			if (get_uint(argv[i]+end, &qos->max_jobs_pu,
 			    "MaxJobs") == SLURM_SUCCESS)
 				set = 1;
 		} else if (!strncasecmp (argv[i], "MaxNodesPerJob",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
-			if (get_uint(argv[i]+end,
-			    &qos->max_nodes_pj,
-			    "MaxNodes") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "MaxNodesPerJob") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_NODE, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_pj, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "MaxNodesPerUser",
 					 MAX(command_len, 8))) {
-			if (!qos)
-				continue;
-			if (get_uint(argv[i]+end,
-			    &qos->max_nodes_pu,
-			    "MaxNodesPerUser") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "MaxNodesPerUser") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_NODE, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_pu, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "MaxSubmitJobsPerUser",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
 			if (get_uint(argv[i]+end, &qos->max_submit_jobs_pu,
 			    "MaxSubmitJobs") == SLURM_SUCCESS)
 				set = 1;
+		} else if (!strncasecmp(argv[i], "MaxTRESPerJob",
+					MAX(command_len, 7))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(
+					db_conn, &tres_cond);
+			}
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_pj, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
+		} else if (!strncasecmp(argv[i], "MaxTRESPerNode",
+					MAX(command_len, 11))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(
+					db_conn, &tres_cond);
+			}
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_pn, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
+		} else if (!strncasecmp(argv[i], "MaxTRESPerUser",
+					MAX(command_len, 11))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(
+					db_conn, &tres_cond);
+			}
+
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_pu, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
+		} else if (!strncasecmp(argv[i], "MaxTRESMinsPerJob",
+					MAX(command_len, 8))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(
+					db_conn, &tres_cond);
+			}
+
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_mins_pj, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
+		} else if (!strncasecmp(argv[i], "MaxTRESRunMins",
+					MAX(command_len, 8))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(
+					db_conn, &tres_cond);
+			}
+
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->max_tres_run_mins_pu, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "MaxWallDurationPerJob",
 					 MAX(command_len, 4))) {
-			if (!qos)
-				continue;
 			mins = time_str2mins(argv[i]+end);
 			if (mins != NO_VAL) {
 				qos->max_wall_pj = (uint32_t) mins;
@@ -435,33 +615,47 @@ static int _set_rec(int *start, int argc, char *argv[],
 			}
 		} else if (!strncasecmp (argv[i], "MinCPUsPerJob",
 					 MAX(command_len, 7))) {
-			if (!qos)
-				continue;
-			if (get_uint(argv[i]+end, &qos->min_cpus_pj,
-			    "MinCPUs") == SLURM_SUCCESS)
+			if (get_uint64(argv[i]+end, &tmp64,
+				       "MinCPUs") == SLURM_SUCCESS) {
 				set = 1;
+				tmp_char = xstrdup_printf(
+					"%d=%"PRIu64, TRES_CPU, tmp64);
+				slurmdb_combine_tres_strings(
+					&qos->min_tres_pj, tmp_char,
+					tres_flags);
+				xfree(tmp_char);
+			}
+		} else if (!strncasecmp(argv[i], "MinTRESPerJob",
+					MAX(command_len, 7))) {
+			if (!g_tres_list) {
+				slurmdb_tres_cond_t tres_cond;
+				memset(&tres_cond, 0,
+				       sizeof(slurmdb_tres_cond_t));
+				tres_cond.with_deleted = 1;
+				g_tres_list = slurmdb_tres_get(
+					db_conn, &tres_cond);
+			}
+			if ((tmp_char = slurmdb_format_tres_str(
+				     argv[i]+end, g_tres_list, 1))) {
+				slurmdb_combine_tres_strings(
+					&qos->min_tres_pj, tmp_char,
+					tres_flags);
+				set = 1;
+				xfree(tmp_char);
+			}
 		} else if (!strncasecmp (argv[i], "PreemptMode",
 					 MAX(command_len, 8))) {
-			if (!qos)
-				continue;
 			qos->preempt_mode = preempt_mode_num(argv[i]+end);
 			if (qos->preempt_mode == (uint16_t)NO_VAL) {
 				fprintf(stderr,
 					" Bad Preempt Mode given: %s\n",
 					argv[i]);
 				exit_code = 1;
-			} else if (qos->preempt_mode == PREEMPT_MODE_SUSPEND) {
-				printf("PreemptType and PreemptMode "
-					"values incompatible\n");
-				exit_code = 1;
 			} else
 				set = 1;
 		/* Preempt needs to follow PreemptMode */
 		} else if (!strncasecmp (argv[i], "Preempt",
 					 MAX(command_len, 7))) {
-			if (!qos)
-				continue;
-
 			if (!qos->preempt_list)
 				qos->preempt_list =
 					list_create(slurm_destroy_char);
@@ -478,18 +672,13 @@ static int _set_rec(int *start, int argc, char *argv[],
 				exit_code = 1;
 		} else if (!strncasecmp (argv[i], "Priority",
 					 MAX(command_len, 3))) {
-			if (!qos)
-				continue;
-
 			if (get_uint(argv[i]+end, &qos->priority,
 			    "Priority") == SLURM_SUCCESS)
 				set = 1;
 		} else if (!strncasecmp (argv[i], "RawUsage",
 					 MAX(command_len, 7))) {
 			uint32_t usage;
-			if (!qos)
-				continue;
-			qos->usage = xmalloc(sizeof(assoc_mgr_qos_usage_t));
+			qos->usage = xmalloc(sizeof(slurmdb_qos_usage_t));
 			if (get_uint(argv[i]+end, &usage,
 				     "RawUsage") == SLURM_SUCCESS) {
 				qos->usage->usage_raw = usage;
@@ -497,16 +686,11 @@ static int _set_rec(int *start, int argc, char *argv[],
 			}
 		} else if (!strncasecmp (argv[i], "UsageFactor",
 					 MAX(command_len, 6))) {
-			if (!qos)
-				continue;
-
 			if (get_double(argv[i]+end, &qos->usage_factor,
 			    "UsageFactor") == SLURM_SUCCESS)
 				set = 1;
 		} else if (!strncasecmp (argv[i], "UsageThreshold",
 					 MAX(command_len, 6))) {
-			if (!qos)
-				continue;
 			if (get_double(argv[i]+end, &qos->usage_thres,
 			    "UsageThreshold") == SLURM_SUCCESS)
 				set = 1;
@@ -526,8 +710,8 @@ static int _set_rec(int *start, int argc, char *argv[],
 static bool _isdefault(List qos_list)
 {
 	int rc = 0;
-	slurmdb_association_cond_t assoc_cond;
-	slurmdb_association_rec_t *assoc = NULL;
+	slurmdb_assoc_cond_t assoc_cond;
+	slurmdb_assoc_rec_t *assoc = NULL;
 	ListIterator itr;
 	List ret_list = NULL;
 	char *name = NULL;
@@ -539,7 +723,7 @@ static bool _isdefault(List qos_list)
 	   can figure out things correctly */
 	xassert(g_qos_list);
 
-	memset(&assoc_cond, 0, sizeof(slurmdb_association_cond_t));
+	memset(&assoc_cond, 0, sizeof(slurmdb_assoc_cond_t));
 	assoc_cond.without_parent_info = 1;
 	assoc_cond.def_qos_id_list = list_create(slurm_destroy_char);
 
@@ -553,9 +737,9 @@ static bool _isdefault(List qos_list)
 	}
 	list_iterator_destroy(itr);
 
-	ret_list = acct_storage_g_get_associations(
+	ret_list = acct_storage_g_get_assocs(
 		db_conn, my_uid, &assoc_cond);
-	list_destroy(assoc_cond.def_qos_id_list);
+	FREE_NULL_LIST(assoc_cond.def_qos_id_list);
 
 	if (!ret_list || !list_count(ret_list))
 		goto end_it;
@@ -587,8 +771,7 @@ static bool _isdefault(List qos_list)
 	list_iterator_destroy(itr);
 	rc = 1;
 end_it:
-	if (ret_list)
-		list_destroy(ret_list);
+	FREE_NULL_LIST(ret_list);
 
 	return rc;
 }
@@ -607,7 +790,7 @@ extern int sacctmgr_add_qos(int argc, char *argv[])
 	List qos_list = NULL;
 	char *qos_str = NULL;
 
-	slurmdb_init_qos_rec(start_qos, 0);
+	slurmdb_init_qos_rec(start_qos, 0, NO_VAL);
 
 	for (i=0; i<argc; i++) {
 		int command_len = strlen(argv[i]);
@@ -619,11 +802,11 @@ extern int sacctmgr_add_qos(int argc, char *argv[])
 	}
 
 	if (exit_code) {
-		list_destroy(name_list);
+		FREE_NULL_LIST(name_list);
 		xfree(description);
 		return SLURM_ERROR;
 	} else if (!list_count(name_list)) {
-		list_destroy(name_list);
+		FREE_NULL_LIST(name_list);
 		slurmdb_destroy_qos_rec(start_qos);
 		exit_code=1;
 		fprintf(stderr, " Need name of qos to add.\n");
@@ -638,7 +821,7 @@ extern int sacctmgr_add_qos(int argc, char *argv[])
 			fprintf(stderr, " Problem getting qos's "
 				"from database.  "
 				"Contact your admin.\n");
-			list_destroy(name_list);
+			FREE_NULL_LIST(name_list);
 			xfree(description);
 			return SLURM_ERROR;
 		}
@@ -651,7 +834,7 @@ extern int sacctmgr_add_qos(int argc, char *argv[])
 		qos = NULL;
 		if (!sacctmgr_find_qos_from_list(g_qos_list, name)) {
 			qos = xmalloc(sizeof(slurmdb_qos_rec_t));
-			slurmdb_init_qos_rec(qos, 0);
+			slurmdb_init_qos_rec(qos, 0, NO_VAL);
 			qos->name = xstrdup(name);
 			if (start_qos->description)
 				qos->description =
@@ -659,50 +842,16 @@ extern int sacctmgr_add_qos(int argc, char *argv[])
 			else
 				qos->description = xstrdup(name);
 
-			qos->flags = start_qos->flags;
-			qos->grace_time = start_qos->grace_time;
-			qos->grp_cpu_mins = start_qos->grp_cpu_mins;
-			qos->grp_cpu_run_mins = start_qos->grp_cpu_run_mins;
-			qos->grp_cpus = start_qos->grp_cpus;
-			qos->grp_jobs = start_qos->grp_jobs;
-			qos->grp_mem = start_qos->grp_mem;
-			qos->grp_nodes = start_qos->grp_nodes;
-			qos->grp_submit_jobs = start_qos->grp_submit_jobs;
-			qos->grp_wall = start_qos->grp_wall;
-
-			qos->max_cpu_mins_pj = start_qos->max_cpu_mins_pj;
-			qos->max_cpu_run_mins_pu =
-				start_qos->max_cpu_run_mins_pu;
-			qos->max_cpus_pj = start_qos->max_cpus_pj;
-			qos->max_cpus_pu = start_qos->max_cpus_pu;
-			qos->max_jobs_pu = start_qos->max_jobs_pu;
-			qos->max_nodes_pj = start_qos->max_nodes_pj;
-			qos->max_nodes_pu = start_qos->max_nodes_pu;
-			qos->max_submit_jobs_pu = start_qos->max_submit_jobs_pu;
-			qos->max_wall_pj = start_qos->max_wall_pj;
-
-			qos->min_cpus_pj = start_qos->min_cpus_pj;
-
-			qos->preempt_list =
-				copy_char_list(start_qos->preempt_list);
-			qos->preempt_mode = start_qos->preempt_mode;
-
-			qos->priority = start_qos->priority;
-
-			qos->usage_factor = start_qos->usage_factor;
-			qos->usage_thres = start_qos->usage_thres;
+			slurmdb_copy_qos_rec_limits(qos, start_qos);
 
 			xstrfmtcat(qos_str, "  %s\n", name);
 			list_append(qos_list, qos);
 		}
 	}
 	list_iterator_destroy(itr);
-	list_destroy(name_list);
+	FREE_NULL_LIST(name_list);
 
-	if (g_qos_list) {
-		list_destroy(g_qos_list);
-		g_qos_list = NULL;
-	}
+	FREE_NULL_LIST(g_qos_list);
 
 	if (!list_count(qos_list)) {
 		printf(" Nothing new added.\n");
@@ -746,7 +895,7 @@ extern int sacctmgr_add_qos(int argc, char *argv[])
 	}
 
 end_it:
-	list_destroy(qos_list);
+	FREE_NULL_LIST(qos_list);
 	xfree(description);
 
 	return rc;
@@ -778,25 +927,25 @@ extern int sacctmgr_list_qos(int argc, char *argv[])
 
 	if (exit_code) {
 		slurmdb_destroy_qos_cond(qos_cond);
-		list_destroy(format_list);
+		FREE_NULL_LIST(format_list);
 		return SLURM_ERROR;
 	} else if (!list_count(format_list)) {
 		slurm_addto_char_list(format_list,
 				      "Name,Prio,GraceT,Preempt,PreemptM,"
 				      "Flags%40,UsageThres,UsageFactor,"
-				      "GrpCPUs,GrpCPUMins,GrpCPURunMins,"
-				      "GrpJ,GrpMEM,GrpN,GrpS,GrpW,"
-				      "MaxCPUs,MaxCPUMins,MaxN,MaxW,"
-				      "MaxCPUsPerUser,"
-				      "MaxJobsPerUser,MaxNodesPerUser,"
-				      "MaxSubmitJobsPerUser,MinCPUs");
+				      "GrpTRES,GrpTRESMins,GrpTRESRunMins,"
+				      "GrpJ,GrpS,GrpW,"
+				      "MaxTRES,MaxTRESPerN,MaxTRESMins,MaxW,"
+				      "MaxTRESPerUser,"
+				      "MaxJobsPerUser,"
+				      "MaxSubmitJobsPerUser,MinTRES");
 	}
 
 	print_fields_list = sacctmgr_process_format_list(format_list);
-	list_destroy(format_list);
+	FREE_NULL_LIST(format_list);
 
 	if (exit_code) {
-		list_destroy(print_fields_list);
+		FREE_NULL_LIST(print_fields_list);
 		return SLURM_ERROR;
 	}
 	qos_list = acct_storage_g_get_qos(db_conn, my_uid, qos_cond);
@@ -805,7 +954,7 @@ extern int sacctmgr_list_qos(int argc, char *argv[])
 	if (!qos_list) {
 		exit_code=1;
 		fprintf(stderr, " Problem with query.\n");
-		list_destroy(print_fields_list);
+		FREE_NULL_LIST(print_fields_list);
 		return SLURM_ERROR;
 	}
 	itr = list_iterator_create(qos_list);
@@ -847,19 +996,39 @@ extern int sacctmgr_list_qos(int argc, char *argv[])
 			case PRINT_GRPCM:
 				field->print_routine(
 					field,
-					qos->grp_cpu_mins,
+					slurmdb_find_tres_count_in_string(
+						qos->grp_tres_mins, TRES_CPU),
 					(curr_inx == field_count));
 				break;
 			case PRINT_GRPCRM:
 				field->print_routine(
 					field,
-					qos->grp_cpu_run_mins,
+					slurmdb_find_tres_count_in_string(
+						qos->grp_tres_run_mins,
+						TRES_CPU),
 					(curr_inx == field_count));
 				break;
 			case PRINT_GRPC:
-				field->print_routine(field,
-						     qos->grp_cpus,
-						     (curr_inx == field_count));
+				field->print_routine(
+					field,
+					slurmdb_find_tres_count_in_string(
+						qos->grp_tres, TRES_CPU),
+					(curr_inx == field_count));
+				break;
+			case PRINT_GRPTM:
+				field->print_routine(
+					field, qos->grp_tres_mins,
+					(curr_inx == field_count));
+				break;
+			case PRINT_GRPTRM:
+				field->print_routine(
+					field, qos->grp_tres_run_mins,
+					(curr_inx == field_count));
+				break;
+			case PRINT_GRPT:
+				field->print_routine(
+					field, qos->grp_tres,
+					(curr_inx == field_count));
 				break;
 			case PRINT_GRPJ:
 				field->print_routine(field,
@@ -867,14 +1036,18 @@ extern int sacctmgr_list_qos(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_GRPMEM:
-				field->print_routine(field,
-						     qos->grp_mem,
-						     (curr_inx == field_count));
+				field->print_routine(
+					field,
+					slurmdb_find_tres_count_in_string(
+						qos->grp_tres, TRES_MEM),
+					(curr_inx == field_count));
 				break;
 			case PRINT_GRPN:
-				field->print_routine(field,
-						     qos->grp_nodes,
-						     (curr_inx == field_count));
+				field->print_routine(
+					field,
+					slurmdb_find_tres_count_in_string(
+						qos->grp_tres, TRES_NODE),
+					(curr_inx == field_count));
 				break;
 			case PRINT_GRPS:
 				field->print_routine(field,
@@ -895,24 +1068,57 @@ extern int sacctmgr_list_qos(int argc, char *argv[])
 			case PRINT_MAXCM:
 				field->print_routine(
 					field,
-					qos->max_cpu_mins_pj,
+					slurmdb_find_tres_count_in_string(
+						qos->max_tres_mins_pj,
+						TRES_CPU),
 					(curr_inx == field_count));
 				break;
 			case PRINT_MAXCRM:
 				field->print_routine(
 					field,
-					qos->max_cpu_run_mins_pu,
+					slurmdb_find_tres_count_in_string(
+						qos->max_tres_run_mins_pu,
+						TRES_CPU),
 					(curr_inx == field_count));
 				break;
 			case PRINT_MAXC:
-				field->print_routine(field,
-						     qos->max_cpus_pj,
-						     (curr_inx == field_count));
+				field->print_routine(
+					field,
+					slurmdb_find_tres_count_in_string(
+						qos->max_tres_pj, TRES_CPU),
+					(curr_inx == field_count));
 				break;
 			case PRINT_MAXCU:
-				field->print_routine(field,
-						     qos->max_cpus_pu,
-						     (curr_inx == field_count));
+				field->print_routine(
+					field,
+					slurmdb_find_tres_count_in_string(
+						qos->max_tres_pu, TRES_CPU),
+					(curr_inx == field_count));
+				break;
+			case PRINT_MAXTM:
+				field->print_routine(
+					field, qos->max_tres_mins_pj,
+					(curr_inx == field_count));
+				break;
+			case PRINT_MAXTRM:
+				field->print_routine(
+					field, qos->max_tres_run_mins_pu,
+					(curr_inx == field_count));
+				break;
+			case PRINT_MAXT:
+				field->print_routine(
+					field, qos->max_tres_pj,
+					(curr_inx == field_count));
+				break;
+			case PRINT_MAXTN:
+				field->print_routine(
+					field, qos->max_tres_pn,
+					(curr_inx == field_count));
+				break;
+			case PRINT_MAXTU:
+				field->print_routine(
+					field, qos->max_tres_pu,
+					(curr_inx == field_count));
 				break;
 			case PRINT_MAXJ:
 				field->print_routine(field,
@@ -920,14 +1126,18 @@ extern int sacctmgr_list_qos(int argc, char *argv[])
 						     (curr_inx == field_count));
 				break;
 			case PRINT_MAXN:
-				field->print_routine(field,
-						     qos->max_nodes_pj,
-						     (curr_inx == field_count));
+				field->print_routine(
+					field,
+					slurmdb_find_tres_count_in_string(
+						qos->max_tres_pj, TRES_NODE),
+					(curr_inx == field_count));
 				break;
 			case PRINT_MAXNU:
-				field->print_routine(field,
-						     qos->max_nodes_pu,
-						     (curr_inx == field_count));
+				field->print_routine(
+					field,
+					slurmdb_find_tres_count_in_string(
+						qos->max_tres_pu, TRES_NODE),
+					(curr_inx == field_count));
 				break;
 			case PRINT_MAXS:
 				field->print_routine(field,
@@ -941,9 +1151,16 @@ extern int sacctmgr_list_qos(int argc, char *argv[])
 					(curr_inx == field_count));
 				break;
 			case PRINT_MINC:
-				field->print_routine(field,
-						     qos->min_cpus_pj,
-						     (curr_inx == field_count));
+				field->print_routine(
+					field,
+					slurmdb_find_tres_count_in_string(
+						qos->min_tres_pj, TRES_CPU),
+					(curr_inx == field_count));
+				break;
+			case PRINT_MINT:
+				field->print_routine(
+					field, qos->min_tres_pj,
+					(curr_inx == field_count));
 				break;
 			case PRINT_NAME:
 				field->print_routine(
@@ -995,8 +1212,8 @@ extern int sacctmgr_list_qos(int argc, char *argv[])
 	}
 	list_iterator_destroy(itr2);
 	list_iterator_destroy(itr);
-	list_destroy(qos_list);
-	list_destroy(print_fields_list);
+	FREE_NULL_LIST(qos_list);
+	FREE_NULL_LIST(print_fields_list);
 
 	return rc;
 }
@@ -1010,15 +1227,15 @@ extern int sacctmgr_modify_qos(int argc, char *argv[])
 	int cond_set = 0, rec_set = 0, set = 0;
 	List ret_list = NULL;
 
-	slurmdb_init_qos_rec(qos, 0);
+	slurmdb_init_qos_rec(qos, 0, NO_VAL);
 
 	for (i=0; i<argc; i++) {
 		int command_len = strlen(argv[i]);
-		if (!strncasecmp (argv[i], "Where", MAX(command_len, 5))) {
+		if (!strncasecmp(argv[i], "Where", MAX(command_len, 5))) {
 			i++;
 			cond_set += _set_cond(&i, argc, argv, qos_cond, NULL);
 
-		} else if (!strncasecmp (argv[i], "Set", MAX(command_len, 3))) {
+		} else if (!strncasecmp(argv[i], "Set", MAX(command_len, 3))) {
 			i++;
 			rec_set += _set_rec(&i, argc, argv, NULL, qos);
 		} else {
@@ -1081,8 +1298,7 @@ extern int sacctmgr_modify_qos(int argc, char *argv[])
 		rc = SLURM_ERROR;
 	}
 
-	if (ret_list)
-		list_destroy(ret_list);
+	FREE_NULL_LIST(ret_list);
 
 	notice_thread_fini();
 
@@ -1182,8 +1398,7 @@ extern int sacctmgr_delete_qos(int argc, char *argv[])
 	}
 
 end_it:
-	if (ret_list)
-		list_destroy(ret_list);
+	FREE_NULL_LIST(ret_list);
 
 	return rc;
 }
