@@ -764,19 +764,6 @@ _wait_for_children_slurmstepd(stepd_step_rec_t *job)
 	pthread_mutex_unlock(&step_complete.lock);
 }
 
-/* If accounting by the job is minimal (i.e. just our "sleep"), then don't
- * send accounting information to slurmctld */
-static bool _minimal_acctg(stepd_step_rec_t *job)
-{
-	if (!job->jobacct)			/* No accounting data */
-		return true;
-	if ((job->jobacct->sys_cpu_sec == 0) &&	/* No measurable usage */
-	    (job->jobacct->user_cpu_sec == 0))
-		return true;
-
-	return false;
-}
-
 /*
  * Send a single step completion message, which represents a single range
  * of complete job step nodes.
@@ -796,9 +783,6 @@ _one_step_complete_msg(stepd_step_rec_t *job, int first, int last)
 
 	debug2("_one_step_complete_msg: first=%d, last=%d", first, last);
 
-	if ((job->stepid == SLURM_EXTERN_CONT) && _minimal_acctg(job))
-		return;
-
 	if (job->batch) {	/* Nested batch step anomalies */
 		if (first == -1)
 			first = 0;
